﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Kurs6
{
	/// <summary>
	/// Okno główne aplikacji
	/// </summary>
	public partial class MainForm : Form
	{
		private TDirectX directx;
		private int startTickCount = Environment.TickCount;

		public MainForm()
		{
			InitializeComponent();

			//Dwie poniższe linie konieczne tylko, gdy nie użyto panelu właściwości
			Load += new System.EventHandler(MainFormLoad);
			Paint += new System.Windows.Forms.PaintEventHandler(MainFormPaint);
			//
		}

		void MainFormLoad(object sender, EventArgs e)
		{
			SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);
			directx = new TDirectX(this, true);
		}

		void MainFormPaint(object sender, PaintEventArgs e)
		{
			if (directx != null)
			{
				directx.Render(Environment.TickCount - startTickCount);
			}
			Invalidate();
		}
	}

	public class TDirectX
	{
		private Device device;
		private Control parent;
		private CustomVertex.PositionColored[] vertex_axis;
		private CustomVertex.PositionTextured[] vertex;
		private VertexBuffer vertexbuffer;
		private Texture texture;

		/// <summary>
		/// Konstruktor obiektu obsługującego DirectX
		/// </summary>
		/// <param name="parent">Okno, które będzie celem renderowania grafiki</param>
		/// <param name="windowed">Czy renderować w oknie (true) czy pełnoekranowo (false)?</param>
		public TDirectX(Control parent, bool windowed)
		{
			this.parent = parent;

			PresentParameters presentParameters = new PresentParameters();
			presentParameters.Windowed = windowed;
			presentParameters.SwapEffect = SwapEffect.Discard;
			presentParameters.AutoDepthStencilFormat = DepthFormat.D16;
			presentParameters.EnableAutoDepthStencil = true;
			if (!windowed)
			{
				presentParameters.FullScreenRefreshRateInHz = 60;
				presentParameters.BackBufferWidth = 800;
				presentParameters.BackBufferHeight = 600;
				presentParameters.BackBufferCount = 1;
				presentParameters.BackBufferFormat = Format.X8R8G8B8;
				presentParameters.PresentFlag = PresentFlag.None;
			}

			device = new Device(0, DeviceType.Hardware, parent, CreateFlags.HardwareVertexProcessing, presentParameters);
			device.DeviceLost += new EventHandler(this.InvalidateDeviceObjects);
			device.DeviceReset += new EventHandler(this.RestoreDeviceObjects);
			device.Disposing += new EventHandler(this.DeleteDeviceObjects);
			device.DeviceResizing += new CancelEventHandler(this.EnvironmentResizing);
			Initialize();
		}

		/// <summary>
		/// Tworzenie wszystkich elementów sceny
		/// </summary>
		private void Initialize()
		{
			#region Teksturowany prostokąt
			vertex = new CustomVertex.PositionTextured[4];

			vertex[0].X=0.1f;
			vertex[0].Y=0.9f;
			vertex[0].Z=0f;
			vertex[0].Tu = 0;
			vertex[0].Tv = 0;

			vertex[1].X=0.9f;
			vertex[1].Y=0.9f;
			vertex[1].Z=0f;
			vertex[1].Tu = 1f;
			vertex[1].Tv = 0;

			vertex[2].X=0.1f;
			vertex[2].Y=0.1f;
			vertex[2].Z=0f;
			vertex[2].Tu = 0;
			vertex[2].Tv = 1f;

			vertex[3].X=0.9f;
			vertex[3].Y=0.1f;
			vertex[3].Z=0f;
			vertex[3].Tu = 1f;
			vertex[3].Tv = 1f;

			vertexbuffer = new VertexBuffer(typeof(CustomVertex.PositionTextured), vertex.Length, device, Usage.Dynamic, CustomVertex.PositionTextured.Format, Pool.Default);
			vertexbuffer.SetData(vertex, 0, LockFlags.None);
			#endregion

			#region Osie układu współrzędnych
			vertex_axis = new CustomVertex.PositionColored[6];

			//Oś X
			vertex_axis[0].X=0f;
			vertex_axis[0].Y=0f;
			vertex_axis[0].Z=0f;
			vertex_axis[0].Color = Color.White.ToArgb();

			vertex_axis[1].X=1f;
			vertex_axis[1].Y=0f;
			vertex_axis[1].Z=0f;
			vertex_axis[1].Color = Color.Red.ToArgb();

			//Oś Y
			vertex_axis[2].X=0f;
			vertex_axis[2].Y=0f;
			vertex_axis[2].Z=0f;
			vertex_axis[2].Color = Color.White.ToArgb();

			vertex_axis[3].X=0f;
			vertex_axis[3].Y=1f;
			vertex_axis[3].Z=0f;
			vertex_axis[3].Color = Color.Green.ToArgb();

			//Oś X
			vertex_axis[4].X=0f;
			vertex_axis[4].Y=0f;
			vertex_axis[4].Z=0f;
			vertex_axis[4].Color = Color.White.ToArgb();			

			vertex_axis[5].X=0f;
			vertex_axis[5].Y=0f;
			vertex_axis[5].Z=1f;
			vertex_axis[5].Color = Color.Blue.ToArgb();
			#endregion

			System.IO.Stream stream = new System.IO.FileStream(@"c:\windows\kawa.bmp", System.IO.FileMode.Open);
			texture = new Texture(device, stream, Usage.Dynamic, Pool.Default);
			stream.Dispose();
		}

		/// <summary>
		/// Wykonanie renderowania
		/// </summary>
		/// <param name="time">Czas działania aplikacji</param>
		public void Render(float time)
		{
			device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Black, 1.0f, 0);
			device.BeginScene();
			device.RenderState.Lighting = false;
			device.RenderState.CullMode = Cull.None;

			device.SetTexture(0, texture);
			device.VertexFormat = CustomVertex.PositionTextured.Format;

			device.SetStreamSource(0, vertexbuffer, 0);
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 2);

			device.VertexFormat = CustomVertex.PositionColored.Format;
			device.DrawUserPrimitives(PrimitiveType.LineList, 3, vertex_axis);

			device.EndScene();
			device.Present();
		}

		#region Obsługa zdarzeń
		/// <summary>
		/// Obsługa zdarzenia utraty dostępu do urządzenia
		/// </summary>
		protected virtual void InvalidateDeviceObjects(object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Obsługa zdarzenia przywrócenia urządzenia
		/// </summary>
		protected virtual void RestoreDeviceObjects(object sender, EventArgs e)
		{
			texture.Dispose();
			Initialize();
		}

		/// <summary>
		/// Obsługa zdarzenia usunięcia urządzenia
		/// </summary>
		protected virtual void DeleteDeviceObjects(object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Obsługa zdarzenia zmiany wymiarów obrazu
		/// </summary>
		protected virtual void EnvironmentResizing(object sender, CancelEventArgs e)
		{
		}
		#endregion
	}
}
